module NdSync
  module Local
    extend ActiveSupport::Concern

    included do
      include NdTranscriber

      scope :pending,   -> { where('step = desired_step - 1') }
      scope :synched,   -> { where('step = desired_step') }

      def model_object
        model_class.find(origin_id)
      end

      def model_class
        @model_class ||= Class.new(ActiveRecord::Base)
        @model_class.table_name  = origin_table
        @model_class.primary_key = origin_id_field
        @model_class
      end

      def transmitted_to web
        updates = {}

        if web.try(:errors).try(:any?)
          updates[origin_step_field] = -1
        else
          updates[origin_step_field] = desired_step
          updates[:web_id] = web.id if web.id && respond_to?(:web_id)
        end

        model_class.where(origin_id_field => origin_id).update_all(updates)
      end
    end

    module ClassMethods
      def to_transmit
        pending.to_a
      end

      def receive(collection)
        collection.each do |remote|
          local = transcribe(remote)
          local.save
          remote.transmitted_to(local)
        end
      end
    end
  end
end
