module NdSync
  module Remote
    extend ActiveSupport::Concern

    included do
      include NdTranscriber

      def transmitted_to(local)
        put("synched/#{local.id}")
      end

      self.site = NdSync.remote_site
    end

    module ClassMethods
      def to_transmit
        find(:all, params: { sync_pending: true }).to_a
      end

      def receive(locals)
        JSON.parse(post_batch(locals).body).each_with_index do |remote_hash, idx|
          locals[idx].transmitted_to new(remote_hash)
        end
      end

      def post_batch(locals)
        post(:batch, {}, to_batch(transcribe_all(locals)))
      end

      def to_batch(collection)
        Jbuilder.encode do |json|
          json.set! collection_name do
            json.array! collection do |item|
              item.to_batch json
            end
          end
        end
      end
    end
  end
end
